import { literal } from '@rdfjs/data-model';
import xsd from './syntax/xsd';
export class SerializationStrategy {
    evaluateTerm(value, options) {
        switch (value.termType) {
            case 'Literal':
                return this.evaluateLiteral(value, options);
            case 'NamedNode':
                return this.evaluateNamedNode(value, options);
            case 'BlankNode':
                return this.evaluateBlankNode(value);
            case 'Variable':
                return this.evaluateVariable(value);
        }
        return {
            value: '',
            prefixes: [],
        };
    }
}
export class TemplateResult {
    constructor({ strings, values, tag, strategy, defaultOptions, }) {
        this.strings = strings;
        this.values = values;
        this._tag = tag;
        this.__strategy = strategy;
        this.__defaultOptions = defaultOptions;
    }
    toString(options) {
        let actualOptions = this.__defaultOptions;
        if (options) {
            actualOptions = { ...actualOptions, ...options };
        }
        const { value, prefixes } = this._toPartialString(actualOptions);
        return this.__strategy.getFinalString(value, prefixes, actualOptions);
    }
    _toPartialString(options) {
        const prefixes = new Set();
        const l = this.strings.length - 1;
        let result = '';
        for (let i = 0; i < l; i++) {
            let partialResult = null;
            result += this.strings[i];
            const value = this.values[i];
            if (typeof value === 'undefined' || value === null)
                continue;
            if (typeof value === 'boolean') {
                partialResult = this.__strategy.evaluateLiteral(literal(value.toString(), xsd.boolean), options);
            }
            else if (typeof value === 'number') {
                const datatype = Number.isInteger(value) ? xsd.integer : xsd.decimal;
                partialResult = this.__strategy.evaluateLiteral(literal(value.toString(), datatype), options);
            }
            else if (value instanceof Date) {
                partialResult = this.__strategy.evaluateLiteral(literal(value.toISOString(), xsd.dateTime), options);
            }
            else if (Array.isArray(value)) {
                partialResult = value.reduce((result, item) => this._tag `${result}\n${item}`, this._tag ``)._toPartialString(options);
            }
            else if (typeof value === 'object') {
                if ('_toPartialString' in value) {
                    partialResult = value._toPartialString(options);
                }
                else if ('subject' in value) {
                    partialResult = this.__strategy.evaluateQuad(value, options);
                }
                else if ('match' in value) {
                    partialResult = this.__strategy.evaluateDataset(value, options);
                }
                else if ('termType' in value) {
                    partialResult = this.__strategy.evaluateTerm(value, options);
                }
            }
            if (partialResult === null) {
                partialResult = {
                    value: value.toString(),
                    prefixes: [],
                };
            }
            result += partialResult.value;
            [...partialResult.prefixes].forEach(prefix => prefixes.add(prefix));
        }
        result += this.strings[l];
        return {
            value: result,
            prefixes,
        };
    }
}
